@@ -19,14 +19,12 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-# ID: $Id: OpenEAFDSS.pl 58 2009-03-19 22:01:46Z hasiotis $
+# ID: $Id: OpenEAFDSS.pl 74 2009-04-02 18:13:59Z hasiotis $
 
 use strict;
-use lib "../lib";
 use Switch;
 use EAFDSS; 
 use Getopt::Std;
-use Data::Dumper; 
 use Config::General qw(ParseConfig);
 
 my(%progie) = ( name      => 'OpenEAFDSS.pl',
@@ -57,7 +55,6 @@ sub main() {
 		case "STATUS"  { cmdStatus($dh)             }
 		case "INFO"    { cmdInfo($dh)               }
 		case "TIME"    { cmdTime($dh, $cmdParam)    }
-		case "QUERY"   { cmdQuery($dh)              }
 		case "HEADERS" { cmdHeaders($dh, $cmdParam) }
 	}
 }
@@ -92,6 +89,7 @@ sub cmdReport() {
 		exit($errNo);
 	}
 }
+
 sub cmdStatus() {
 	my($dh) = shift @_;
 
@@ -152,24 +150,6 @@ sub cmdTime() {
 
 }
 
-sub cmdQuery() {
-	my($dh) = shift @_;
-
-	my($devices) = $dh->Query();
-	my($errNo)  = $dh->error();
-	if ($devices) {
-		while ( my($key, $value) = each %$devices) {
-			printf("%s\n", $key);
-		}
-		exit(0);
-	} else {
-		my($errNo)  = $dh->error();
-		my($errMsg) = $dh->errMessage($errNo);
-		printf(STDERR "ERROR [0x%02X]: %s\n", $errNo, $errMsg);
-		exit($errNo);
-	}
-}
-
 sub cmdHeaders() {
 	my($dh)      = shift @_;
 	my($headers) = shift @_;
@@ -265,7 +245,6 @@ sub print_help() {
 	printf("\t                                         - STATUS                   \n");
 	printf("\t                                         - REPORT                   \n");
 	printf("\t                                         - INFO                     \n");
-	printf("\t                                         - QUERY                    \n");
 	printf("\t                                         - HEADERS [headers])       \n");
 	printf("\n  Example 1: $progie{name} -d EAFDSS::SDNP -p hostname -e \"SIGN invoice.txt\"\n");
 	printf("\n             This command will sign the file invoice.txt printing the signature");
