@@ -3,7 +3,7 @@
 #
 # Copyright (C) 2008 Hasiotis Nikos
 #
-# ID: $Id: Base.pm 58 2009-03-19 22:01:46Z hasiotis $
+# ID: $Id: Base.pm 76 2009-04-02 20:55:44Z hasiotis $
 
 package EAFDSS::Base;
 
@@ -12,11 +12,10 @@ use strict;
 use warnings;
 use Carp;
 use Class::Base;
-use Data::Dumper;
 
 use base qw ( Class::Base );
 
-our($VERSION) = '0.20';
+our($VERSION) = '0.40';
 
 sub init {
 	my($self, $config) = @_;
@@ -43,32 +42,32 @@ sub Sign {
 
         $self->debug("Sign operation");
 
-        if (-e $fname) {
+        if ( ($fname eq '-') || (-e $fname) ) {
 		my($replySignDir, $deviceDir) = $self->_createSignDir();
 		if ($replySignDir != 0) {
 			return $self->error($replySignDir);
 		}
 
-                $self->debug(  "  Checking file [%s]", $fname);
+		# Slurping the invoice
                 open(FH, $fname);
-		my($invalid) = $self->_checkCharacters(*FH);
+	        my($invoice) = do { local($/); <FH> };
                 close(FH);
 
+                $self->debug(  "  Checking file [%s] for invalid characters", $fname);
+		my($invalid) = $self->_checkCharacters($invoice);
                 if ($invalid)  {
 			$self->debug("  File contains invalid characters [%s]", $fname);
 			return $self->error(64+0x10);
 		}
 
                 $self->debug(  "  Signing file [%s]", $fname);
-                open(FH, $fname);
-                ($reply, $totalSigns, $dailySigns, $date, $time, $nextZ, $sign) = $self->PROTO_GetSign(*FH);
-                close(FH);
+                ($reply, $totalSigns, $dailySigns, $date, $time, $nextZ, $sign) = $self->PROTO_GetSign($invoice);
 
 		if ($reply == 0) {
 			$fullSign = sprintf("%s %04d %08d %s%s %s",
 				$sign, $dailySigns, $totalSigns, $self->UTIL_date6ToHost($date), substr($time, 0, 4), $self->{SN});
 
-			$self->_createFileA($fname, $deviceDir, $date, $dailySigns, $nextZ);
+			$self->_createFileA($invoice, $deviceDir, $date, $dailySigns, $nextZ);
 			$self->_createFileB($fullSign, $deviceDir, $date, $dailySigns, $nextZ);
 
 	        	return $fullSign;
@@ -83,13 +82,13 @@ sub Sign {
 }
 
 sub _checkCharacters {
-        my($self) = shift @_;
-	my($fh)   = shift @_;
+        my($self)    = shift @_;
+	my($invoice) = shift @_;
 	
 	my($c);
-	while (read($fh, $c, 1)) {
+	foreach $c (unpack('C*', $invoice)) {
 		if (grep $_ == ord($c), qw/0 1 2 3 4 5 6 7 8 11 14 15 16 17 18 19 20 21 22 23 24 25 27 28 29 30 31 127 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 173 210 255/ ) {
-                	$self->debug("  Found invalid character [%d]", ord($c));
+                	$self->debug("     Found invalid character [%d]", ord($c));
 			return 1;
 		}
 	}
@@ -161,6 +160,27 @@ sub Report {
         return $z;
 }
 
+sub _RecoveryReport {
+        my($self) = shift @_;
+
+	my($deviceDir) = sprintf("%s/%s", $self->{DIR}, $self->{SN});
+
+	$self->_validateFilesB();
+	$self->_validateFilesC();
+
+        $self->debug("Issue Recovery Report operation");
+
+	my($reply1) = $self->PROTO_IssueReport();
+	if ($reply1 != 0) {
+		return $self->error($reply1);
+	}
+
+	my($reply2, $status1, $status2, $totalSigns, $dailySigns, $date, $time, $z, $sn, $closure) = $self->PROTO_ReadClosure(0);
+	$self->_createFileC($z, $deviceDir, $date, $time, $closure);
+
+        return $z;
+}
+
 sub Info {
         my($self) = shift @_;
 
@@ -273,7 +293,6 @@ sub _Recover {
 
 		my($curB)  = $curA; $curB =~ s/_a/_b/;
 		my($curIndex) = substr($curA, 21, 4); $curIndex =~ s/^0*//;
-		$self->debug("            Updating file B  [%s] -- Index [%d]", $curB, $curIndex);
 
 		$self->debug("            Resigning file A [%s]", $curA);
 		open(FH, $curFileA);
@@ -282,34 +301,35 @@ sub _Recover {
 		my($fullSign) = sprintf("%s %04d %08d %s%s %s", $sign, $dailySigns, $totalSigns, $self->UTIL_date6ToHost($date), substr($time, 0, 4), $self->{SN});
 		close(FH);
 
+		$self->debug("            Updating file  B [%s] -- Index [%d]", $curB, $curIndex);
 		open(FB, ">>", $curFileB) || croak "Error: $!";
 		print(FB "\n" . $fullSign); 
 		close(FB);
 	}
 
-	my($replyFinal, $z) = $self->Report();
+	my($z) = $self->_RecoveryReport();
+	if ($z) {
+		return(0);
+	} else {
+		my($errNo) = $self->error();
+		return $self->error($errNo);
+	}
 
-	return($replyFinal);
 }
 
 sub _createFileA {
-	my($self) = shift @_;
-	my($fn)   = shift @_;
-	my($dir)  = shift @_;
-	my($date) = shift @_;
-	my($ds)   = shift @_;
-	my($curZ) = shift @_;
+	my($self)    = shift @_;
+	my($invoice) = shift @_;
+	my($dir)     = shift @_;
+	my($date)    = shift @_;
+	my($ds)      = shift @_;
+	my($curZ)    = shift @_;
 
 	my($fnA) = sprintf("%s/%s%s%04d%04d_a.txt", $dir, $self->{SN}, $self->UTIL_date6ToHost($date), $curZ, $ds);
 	$self->debug("   Creating File A [%s]", $fnA);
-	open(FH, $fn);
 	open(FA, ">", $fnA) || croak "Error: $!";
-	seek(FH, 0, 0);
-	while (<FH>) {
-		print(FA $_);
-	};
+	print(FA $invoice);
 	close(FA);
-	close(FH);
 }
 
 sub _createFileB {
@@ -370,8 +390,8 @@ sub _validateFilesB {
                         my($curIndex) = substr($curA, 21, 4); $curIndex =~ s/^0*//;
                         $self->debug(  "            Recreating file B [%s] -- Index [%d]", $curB, $curIndex);
 
-                        my($replyCode, $status1, $status2, $totalSigns, $dailySigns, $date, $time, $sign, $sn, $closure) = $self->ReadSignEntry($curIndex);
-                        my($fullSign) = sprintf("%s %04d %08d %s%s %s", $sign, $dailySigns, $totalSigns, $self->date6ToHost($date), substr($time, 0, 4), $self->{SN});
+                        my($replyCode, $status1, $status2, $totalSigns, $dailySigns, $date, $time, $sign, $sn, $closure) = $self->PROTO_ReadSignEntry($curIndex);
+                        my($fullSign) = sprintf("%s %04d %08d %s%s %s", $sign, $dailySigns, $totalSigns, $self->UTIL_date6ToHost($date), substr($time, 0, 4), $self->{SN});
 
                         open(FB, ">",  $curFileB) || croak "Error: $!";
                         print(FB $fullSign); 
@@ -464,7 +484,7 @@ Read EAFDSS on how to use the module.
 
 =head1 VERSION
 
-This is version 0.20.
+This is version 0.40.
 
 =head1 AUTHOR
 
