@@ -3,7 +3,7 @@
 #
 # Copyright (C) 2008 Hasiotis Nikos
 #
-# ID: $Id: Micrelec.pm 58 2009-03-19 22:01:46Z hasiotis $
+# ID: $Id: Micrelec.pm 76 2009-04-02 20:55:44Z hasiotis $
 
 package EAFDSS::Micrelec;
 
@@ -13,11 +13,10 @@ use warnings;
 use Switch;
 use Carp;
 use Class::Base;
-use Data::Dumper;
 
 use base qw ( EAFDSS::Base );
 
-our($VERSION) = '0.20';
+our($VERSION) = '0.40';
 
 sub PROTO_DetailSign {
 	my($self) = shift @_;
@@ -26,7 +25,7 @@ sub PROTO_DetailSign {
 	my($reply, $totalSigns, $dailySigns, $date, $time, $sign) = $self->GetSign($fh);
 
 	if ($reply == 0) {
-		return ($reply, sprintf("%s %04d %08d %s%s %s", $sign, $dailySigns, $totalSigns, $self->date6ToHost($date), substr($time, 0, 4), $self->{SN}));
+		return (hex($reply), sprintf("%s %04d %08d %s%s %s", $sign, $dailySigns, $totalSigns, $self->date6ToHost($date), substr($time, 0, 4), $self->{SN}));
 	} else {
 		return (-1);
 	}
@@ -39,15 +38,15 @@ sub PROTO_Query{
 	my($replyCode, $devices) = $self->_sdnpQuery();
 
 	if (! $replyCode) {
-		return ($replyCode, $devices);
+		return (hex($replyCode), $devices);
 	} else {
 		return ($self->error());
 	}
 }
 
 sub PROTO_GetSign {
-	my($self) = shift @_;
-	my($fh)   = shift @_;
+	my($self)    = shift @_;
+	my($invoice) = shift @_;
 
 	my($chunk, %reply);
 	$self->debug("  [PROTO] Get Sign");
@@ -62,7 +61,7 @@ sub PROTO_GetSign {
 		$reply{OPCODE} = 0x22;
 	}
 	if ( %reply && ($reply{OPCODE} == 0x22) ) {
-		while (read($fh, $chunk, 400)) {
+		foreach $chunk (unpack('(A400)*', $invoice)) {
 			my(%reply) = $self->SendRequest(0x21, 0x00, "@/$chunk");
 		}
 		%reply = $self->SendRequest(0x21, 0x00, "}");
@@ -70,7 +69,7 @@ sub PROTO_GetSign {
 
 	if (%reply) { 
 		my($replyCode, $status1, $status2, $totalSigns, $dailySigns, $date, $time, $sign, $sn, $nextZ) = split(/\//, $reply{DATA});
-		return ($replyCode, $totalSigns, $dailySigns, $date, $time, $nextZ, $sign);
+		return (hex($replyCode), $totalSigns, $dailySigns, $date, $time, $nextZ, $sign);
 	} else {
 		return (-1);
 	}
@@ -156,6 +155,10 @@ sub PROTO_SetTime {
 	my($time) = shift @_;
 
 	$self->debug("  [PROTO] Set Time");
+	if (length($time) < 16) {
+		return hex(3);
+	}
+
         my($pdate) = substr($time, 0, 2) . substr($time, 3, 2) . substr($time, 6, 2);
         my($ptime) = substr($time, 9, 2) . substr($time, 12, 2) . substr($time, 15, 2);
 	my(%reply) = $self->SendRequest(0x21, 0x00, "T/$pdate/$ptime/");
@@ -396,7 +399,7 @@ Read EAFDSS on how to use the module.
 
 =head1 VERSION
 
-This is version 0.20.
+This is version 0.40.
 
 =head1 AUTHOR
 
